/*

 wminet.c
 Multi-function system monitor
 by Dave Clark (clarkd@skynet.ca) (http://www.neotokyo.org/illusion)
 Martijn Pieterse (pieterse@xs4all.nl) (http://windowmaker.mezaway.org)
 and Antoine Nulle (warp@xs4all.nl) (http://windowmaker.mezaway.org)

 This software is licensed through the GNU General Public Lisence.

 ProFTPD support by Mike Kershaw aka Dragorn (dragorn@melchior.nerv-un.ml.org)
 ProFTPD support was made 64bit clean by Martijn Pieterse (pieterse@xs4all.nl)
 
 see http://windowmaker.mezaway.org for more awesome wm dock apps :)

*/


#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>

#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <utmp.h>
#include <dirent.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "../wmgeneral/wmgeneral.h"
#include "../wmgeneral/misc.h"

#include "wmscript-master.xpm"
char wminet_mask_bits[64*64];
int  wminet_mask_width = 64;
int  wminet_mask_height = 64;

#define WMSCRIPT_VERSION "1.0"

#define CHAR_WIDTH 5
#define CHAR_HEIGHT 7

// Lame work-around... Sigh... when will they standardize the headers!?!?
#define TCP_ESTABLISHED 1

extern	char **environ;

char	*ProgName;

int loopinterval=1;

typedef struct
{
    int pos;
    char script[256];
    char label[10];
    char action[256];
    int interval;
    int value;
} script_t;
script_t scripts[6];

int timers[6];

char uconfig_file[256];

void usage(void);
void printversion(void);
void BlitString(char *name, int x, int y);
void BlitNum(int num, int x, int y);
void GetStats(int script);
void wminet_routine(int, char **);
int ReadConfigInt(FILE *fp, char *setting, int *value);
int ReadConfigString(FILE *fp, char *setting, char *value);
int Read_Config_File( char *filename );
int RunScript(const char* script);
char *GetAction(const int pos, char *action);
void CreateDisplay();
void debug(const char* buff);
void debugi(const int val);


int main(int argc, char *argv[]) {

	int		i;

    uconfig_file[0] = 0;
    
	/* Parse Command Line */

	ProgName = argv[0];
	if (strlen(ProgName) >= 5)
		ProgName += (strlen(ProgName) - 5);
	
	for (i=1; i<argc; i++) {
		char *arg = argv[i];

		if (*arg=='-') {
			switch (arg[1]) {
			case 'd' :
				if (strcmp(arg+1, "display")) {
					usage();
					exit(1);
				}
				break;
			case 'g' :
				if (strcmp(arg+1, "geometry")) {
					usage();
					exit(1);
				}
				break;
			case 'v' :
				printversion();
				exit(0);
				break;
            case 'c' :
                if (argc > (i+1))
                {
                    strcpy(uconfig_file, argv[i+1]);
                    i++;
                }
                break;
            default:
				usage();
				exit(0);
				break;
			}
		}
	}

	wminet_routine(argc, argv);

	return 0;
}

/*******************************************************************************\
|* wminet_routine														   *|
\*******************************************************************************/

void wminet_routine(int argc, char **argv)
{
    int			i;
	XEvent		Event;
	int			but_stat = -1;

    time_t		curtime;
	time_t		prevtime;

    char config_file[512];

    memset(&scripts, 0, sizeof(scripts));
    memset(&timers, 0, sizeof(timers));
    
	createXBMfromXPM(wminet_mask_bits, wminet_master_xpm, wminet_mask_width, wminet_mask_height);
    
	openXwindow(argc, argv, wminet_master_xpm, wminet_mask_bits, wminet_mask_width, wminet_mask_height);

	AddMouseRegion(0, 5, 6, 58, 16);
	AddMouseRegion(1, 5, 16, 58, 26);
	AddMouseRegion(2, 5, 26, 58, 36);
	AddMouseRegion(3, 5, 36, 58, 46);
	AddMouseRegion(4, 5, 46, 58, 56);

    // Read config file

    if (uconfig_file[0] != 0)
    {
        // user-specified config file
        fprintf(stderr, "Using user-specified config file '%s'.\n", uconfig_file);
        Read_Config_File(uconfig_file);
    }
    else
    {
        sprintf(config_file, "%s/.wmscriptrc", getenv("HOME"));

        if (!Read_Config_File(config_file))
        {
            // Fall back to /etc/wmscriptrc
            sprintf(config_file, "/etc/wmscriptrc");
        
            Read_Config_File(config_file);
        }
    }

    // Initial display
    GetStats(0);
    CreateDisplay();
    RedrawWindow();

    prevtime = time(0) - 1;
    
    while (1)
    {
		curtime = time(0);
		waitpid(0, NULL, WNOHANG);

        if ( curtime >= prevtime + loopinterval)
        {
            int loop;
            prevtime = curtime;

            for (loop = 1;loop<6;loop++)
            {
                timers[loop]++;
                
                if (timers[loop] >= scripts[loop].interval)
                {
                    timers[loop] = 0;
                    // Get data to display
                    GetStats(loop);
                    CreateDisplay();
                    RedrawWindow();
                }
            }
        }
        
        // X Events
        while (XPending(display))
        {
			XNextEvent(display, &Event);
            switch (Event.type)
            {
			case Expose:
				RedrawWindow();
				break;
			case DestroyNotify:
				XCloseDisplay(display);
				exit(0);
                break;
			case ButtonPress:
				i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);

				but_stat = i;
				break;
			case ButtonRelease:
				i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);

                if (but_stat == i && but_stat >= 0)
                {
                    char action[256];
                    execCommand(GetAction(but_stat+1, action));
				}
				but_stat = -1;
//				RedrawWindow();
				break;
			}
		}

		usleep(100000L);
	}
}

// runs the script specified by the number passed in, zero for all
void GetStats(int script)
{
    int loop, loop_start, loop_stop;
    
    if (script == 0)
    {
        loop_start = 1;
        loop_stop = 6;
    }
    else
    {
        loop_start = script;
        loop_stop = script+1;
    }

    for (loop = loop_start; loop<loop_stop;loop++)
    {
        if (scripts[loop].pos != 0)
        {
            char script[256];
            int len, ret;

            len = strlen(scripts[loop].script);

            strncpy(script, scripts[loop].script, len);
            script[len] = '\0';

            ret = RunScript (script);

            scripts[loop].value = ret;

        }
    }
}


// Blits a string at given co-ordinates
void BlitString(char *name, int x, int y)
{
    int		i;
	int		c;
    int		k;

	k = x;
    for (i=0; name[i]; i++)
    {

        c = toupper(name[i]); 
        if (c >= 'A' && c <= 'Z')
        {   // its a letter
			c -= 'A';
			copyXPMArea(c * 6, 74, 6, 8, k, y);
			k += 6;
        }
        else
        {   // its a number or symbol
			c -= '0';
			copyXPMArea(c * 6, 64, 6, 8, k, y);
			k += 6;
		}
	}

}


// Blits number to give coordinates.. two 0's, right justified

void BlitNum(int num, int x, int y)
{
    char buf[1024];
    int newx=x;

    if (num > 99)
    {
        newx -= CHAR_WIDTH;
    }

    if (num > 999)
    {
        num = 999;
    }

    sprintf(buf, "%02i", num);
    
    BlitString(buf, newx, y);
}
    

// ReadConfigSetting
int ReadConfigString(FILE *fp, char *setting, char *value)
{
    char str[1024];
    char buf[1024];
    int i;
    int len;
    int slen;
    char *p=NULL;

    if (!fp)
    {
        return 0;
    }

    sprintf(str, "%s=", setting);
    slen = strlen(str);
    
    fseek(fp, 0, SEEK_SET);

    while ( !feof(fp) )
    {
        
        if (!fgets(buf, 512, fp))
            break;
        
        len = strlen(buf);

        // strip linefeed
        for (i=0; i!=len; i++)
        {
            if (buf[i] == '\n')
            {
                buf[i] = 0;
            }
        }

        //printf("Scanning '%s'...\n", buf);
        if ( strncmp(buf, str, strlen(str)) == 0)
        {
            // found our setting
            
            for(i=0; i!=slen; i++)
            {
                if ( buf[i] == '=' )
                {
                    p=buf+i+1;
                    strcpy(value, p);
                    return 1;
                }
            }
    
        }
    }
    
        return 0;
}

int ReadConfigInt(FILE *fp, char *setting, int *value)
{
    char buf[1024];


    if (ReadConfigString(fp, setting, (char *) &buf))
    {
        *value = atoi(buf);
        return 1;
    }

    return 0;
}

int Read_Config_File( char *filename )
{
    FILE *fp;

    fp = fopen(filename, "r");
    if (fp)
    {
        int loop;

        for (loop = 1; loop<6;loop++)
        {
            char pos[15], script[15], label[15], action[15], interval[17];
            sprintf(pos,"script%d.pos",loop);
            sprintf(script,"script%d.script",loop);
            sprintf(label,"script%d.label",loop);
            sprintf(action,"script%d.action",loop);
            sprintf(interval,"script%d.interval",loop);

            ReadConfigInt(fp, pos, &scripts[loop].pos);
            ReadConfigString(fp, script, scripts[loop].script);
            ReadConfigString(fp, label, scripts[loop].label);
            ReadConfigString(fp, action, scripts[loop].action);
            ReadConfigInt(fp, interval, &scripts[loop].interval);
        }
        
        fclose(fp);
        return 1;
    }
    else
    {
        perror("Read_Config_File");
        fprintf(stderr, "Unable to open %s, no settings read.\n", filename);
        return 0;
    }

}


int RunScript(const char* script)
{
    int ret = -1, loop;
    FILE *proc;
    char buff[256];
    char ch = '\0';
    
    memset (buff, 0, 255);

    proc = popen(script, "r");

    if (proc != NULL)
    {
        for (loop = 0; loop<255 && ch != EOF;loop++)
        {
            ch = getc(proc);
            if (ch != EOF)
                buff[loop] = ch;
        }

        pclose (proc);
    }
    ret = atoi(buff);

    return ret;
}

char *GetAction(const int pos, char *action)
{
    int loop;

    for (loop = 1; loop<6;loop++)
    {

        if (pos == scripts[loop].pos)
        {
            strcpy(action, scripts[loop].action);
        }
    }
    
    return action;

}

/*******************************************************************************\
|* usage																	   *|
\*******************************************************************************/

void usage(void)
{
    fprintf(stderr, "\nWMScript - Darren Greaves <darren@magma.dircon.co.uk>, illusion <clarkd@skyia.com>, warp <warp@xs4all.nl>\n         & tijno <pieterse@xs4all.nl>\n\n");
	fprintf(stderr, "usage:\n");
	fprintf(stderr, "    -display <display name>\n");
	fprintf(stderr, "    -geometry +XPOS+YPOS      initial window position\n");
    fprintf(stderr, "    -c <filename>             use specified config file\n");
    fprintf(stderr, "    -h                        this help screen\n");
	fprintf(stderr, "    -v                        print the version number\n");
	fprintf(stderr, "\n");
}

/*******************************************************************************\
|* printversion																   *|
\*******************************************************************************/

void printversion(void)
{
	fprintf(stderr, "wmscript v%s\n", WMSCRIPT_VERSION);
}

void debug(const char* buff)
{
    fprintf (stderr, buff);
    fprintf (stderr, "\n");
}

void debugi(const int val)
{
    char buff[256];
    
    sprintf (buff, "%d",val);

    fprintf (stderr, buff);
    fprintf (stderr, "\n");
}

void CreateDisplay()
{
    char buf[64];
    int	i,j,k;

    for (i=1; i<6; i++)
    {
        for (j=1; j<6; j++)
        {
            if ( i == scripts[j].pos )
            {
                strncpy(buf, scripts[j].label, 5);

                for (k=0; k<5; k++)
                {
                    if (buf[k] == 0)
                        buf[k] = ' ';
                }
                
                buf[5] = ':';
                buf[6] = 0;

                BlitString(buf, 5, (11*(i-1)) + 5);
                copyXPMArea(39, 84, (3*CHAR_WIDTH), 8, 39, (11*(i-1)) + 5);
                BlitNum(scripts[j].value, 45, (11*(i-1)) + 5);
            }
        }
    }
}
